

class UrlBuilder:

    base_url = 'https://api.fanduel.com'
    contests_path = '/contests'
    fixture_lists_path = '/fixture-lists'

    def __init__(self):
        pass

    @staticmethod
    def build_contest_url():
        return '{}{}'.format(UrlBuilder.base_url, UrlBuilder.contests_path)

    @staticmethod
    def build_fixture_lists_url():
        return '{}{}'.format(UrlBuilder.base_url, UrlBuilder.fixture_lists_path)

    @staticmethod
    def build_fixture_list_players_url(fixture_list_id):
        return '{}/{}/players'.format(UrlBuilder.build_fixture_lists_url(), fixture_list_id)


class HeadersBuilder:

    host_value = 'api.fanduel.com'
    origin_value = 'https://www.fanduel.com'
    referer_value = 'https://www.fanduel.com/games'

    def __init__(self):
        pass

    @staticmethod
    def build(basic_authorization_header_value, x_auth_token_header_value):
        return {
            'Authorization': 'Basic {}'.format(basic_authorization_header_value),
            'X-Auth-Token': x_auth_token_header_value,
            'Host': HeadersBuilder.host_value,
            'Origin': HeadersBuilder.origin_value,
            'Referer': HeadersBuilder.referer_value
        }



from datetime import datetime
from enum import Enum

# TODO: @jbradley not all sports are represented - add sports as they are identified


class Sport(Enum):
    nba = 'nba'
    nhl = 'nhl'
    nfl = 'nfl'
    pga = 'pga'
    epl = 'epl'

    @staticmethod
    def value_of(name):
        assert isinstance(name, basestring)

        for sport in Sport:
            if sport.value == name.lower():
                return sport

        raise ValueError('Unknown sport: %s', name)

# TODO: @jbradley not all fields in Contest JSON are represented - add fields as needed


class Contest:
    def __init__(self, contest_id, max_entries_per_user, name, is_guaranteed, url, entry_fee):
        assert isinstance(contest_id, basestring)
        assert isinstance(max_entries_per_user, int)
        assert isinstance(name, basestring)
        assert isinstance(is_guaranteed, bool)
        assert isinstance(url, basestring)
        assert isinstance(entry_fee, int)

        self.contest_id = contest_id
        self.max_entries_per_user = max_entries_per_user
        self.name = name
        self.is_guaranteed = is_guaranteed
        self.url = url
        self.entry_fee = entry_fee


class FixtureListStatus:
    def __init__(self, has_started, is_final):
        assert isinstance(has_started, bool)
        assert isinstance(is_final, bool)

        self.has_started = has_started
        self.is_final = is_final


class FixtureList:
    def __init__(self, fixture_id, url, status, start_time, players_url, sport, salary_cap):
        assert isinstance(fixture_id, basestring)
        assert isinstance(url, basestring)
        assert isinstance(status, FixtureListStatus)
        assert isinstance(start_time, datetime)
        assert isinstance(players_url, basestring)
        assert isinstance(sport, Sport)
        assert isinstance(salary_cap, int)

        self.fixture_list_id = fixture_id
        self.url = url,
        self.status = status
        self.start_time = start_time
        self.players_url = players_url
        self.sport = sport
        self.salary_cap = salary_cap


class Team(Enum):
    atlanta_hawks = {
        'sport': Sport.nba,
        'id': 679
    }
    boston_celtics = {
        'sport': Sport.nba,
        'id': 680
    }
    brooklyn_nets = {
        'sport': Sport.nba,
        'id': 696
    }
    charlotte_hornets = {
        'sport': Sport.nba,
        'id': 681
    }
    chicago_bulls = {
        'sport': Sport.nba,
        'id': 682
    }
    cleveland_cavaliers = {
        'sport': Sport.nba,
        'id': 683
    }
    dallas_mavericks = {
        'sport': Sport.nba,
        'id': 684
    }
    denver_nuggets = {
        'sport': Sport.nba,
        'id': 685
    }
    detroit_pistons = {
        'sport': Sport.nba,
        'id': 686
    }
    golden_state_warriors = {
        'sport': Sport.nba,
        'id': 687
    }
    houston_rockets = {
        'sport': Sport.nba,
        'id': 688
    }
    indiana_pacers = {
        'sport': Sport.nba,
        'id': 689
    }
    los_angeles_clippers = {
        'sport': Sport.nba,
        'id': 690
    }
    los_angeles_lakers = {
        'sport': Sport.nba,
        'id': 691
    }
    memphis_grizzlies = {
        'sport': Sport.nba,
        'id': 692
    }
    miami_heat = {
        'sport': Sport.nba,
        'id': 693
    }
    milwaukee_bucks = {
        'sport': Sport.nba,
        'id': 694
    }
    minnesota_timberwolves = {
        'sport': Sport.nba,
        'id': 695
    }
    new_orleans_pelicans = {
        'sport': Sport.nba,
        'id': 697
    }
    new_york_knicks = {
        'sport': Sport.nba,
        'id': 698
    }
    oklahoma_city_thunder = {
        'sport': Sport.nba,
        'id': 699
    }
    orlando_magic = {
        'sport': Sport.nba,
        'id': 700
    }
    philadelphia_76ers = {
        'sport': Sport.nba,
        'id': 701
    }
    phoenix_suns = {
        'sport': Sport.nba,
        'id': 702
    }
    portland_trail_blazers = {
        'sport': Sport.nba,
        'id': 703
    }
    sacramento_kings = {
        'sport': Sport.nba,
        'id': 704
    }
    san_antonio_spurs = {
        'sport': Sport.nba,
        'id': 705
    }
    toronto_raptors = {
        'sport': Sport.nba,
        'id': 706
    }
    utah_jazz = {
        'sport': Sport.nba,
        'id': 707
    }
    washington_wizards = {
        'sport': Sport.nba,
        'id': 708
    }

    @staticmethod
    def value_of(sport, fan_duel_id):
        assert isinstance(sport, Sport)
        assert isinstance(fan_duel_id, int)

        for team in Team:
            if team.value['sport'] == sport and team.value['id'] == fan_duel_id:
                return team

        raise ValueError('Unable to identify team for sport: %s and id: %s', sport, fan_duel_id)


class Position(Enum):
    point_guard = {
        'sport': Sport.nba,
        'abbreviation': 'PG',
        'name': 'Point Guard'
    }
    shooting_guard = {
        'sport': Sport.nba,
        'abbreviation': 'SG',
        'name': 'Shooting Guard'
    }
    small_forward = {
        'sport': Sport.nba,
        'abbreviation': 'SF',
        'name': 'Small Forward'
    }
    power_forward = {
        'sport': Sport.nba,
        'abbreviation': 'PF',
        'name': 'Power Forward'
    }
    center = {
        'sport': Sport.nba,
        'abbreviation': 'C',
        'name': 'Center'
    }

    @staticmethod
    def value_of(sport, abbreviation):
        assert isinstance(sport, Sport)
        assert isinstance(abbreviation, basestring)

        for position in Position:
            if position.value['sport'] == sport and position.value['abbreviation'] == abbreviation:
                return position

        raise ValueError('Unable to identify position for sport: %s and abbreviation: %s', sport, abbreviation)


class FixtureStatus:
    def __init__(self, is_final, has_started):
        assert isinstance(is_final, bool)
        assert isinstance(has_started, bool)

        self.is_final = is_final
        self.has_started = has_started


class Fixture:
    def __init__(self, fixture_id, sport, status, away_team, home_team, start_time):
        assert isinstance(fixture_id, int)
        assert isinstance(sport, Sport)
        assert isinstance(status, FixtureStatus)
        assert isinstance(away_team, Team)
        assert isinstance(home_team, Team)
        assert isinstance(start_time, datetime)

        self.fixture_id = fixture_id
        self.sport = sport
        self.status = status
        self.away_team = away_team
        self.home_team = home_team
        self.status = status
        self.start_time = start_time


class FixturePlayerInjury:
    def __init__(self, details, status):
        assert isinstance(details, (type(None), basestring))
        assert isinstance(status, (type(None), basestring))

        self.details = details
        self.status = status


class FixturePlayer:
    def __init__(self, fixture_player_id, first_name, last_name, jersey_number, injury, is_removed, position, salary, team, fixture):
        assert isinstance(fixture_player_id, basestring)
        assert isinstance(first_name, basestring)
        assert isinstance(last_name, basestring)
        assert isinstance(jersey_number, int)
        assert isinstance(is_removed, bool)
        assert isinstance(injury, FixturePlayerInjury)
        assert isinstance(position, Position)
        assert isinstance(team, Team)
        assert isinstance(fixture, Fixture)
        assert isinstance(salary, int)

        self.fixture_player_id = fixture_player_id
        self.first_name = first_name
        self.last_name = last_name
        self.jersey_number = jersey_number
        self.injury = injury
        self.is_removed = is_removed
        self.position = position
        self.salary = salary
        self.team = team
        self.fixture = fixture



import requests

from data.deserializers import ContestsDeserializer, FixtureListsDeserializer, FixturePlayersDeserializer
from utilities.builders import UrlBuilder, HeadersBuilder


class FanDuelClient:
    def __init__(self, basic_authorization_header_value, x_auth_token_header_value):
        self.headers = HeadersBuilder.build(basic_authorization_header_value=basic_authorization_header_value,
                                            x_auth_token_header_value=x_auth_token_header_value)

    def get_contests(self):
        response = requests.get(url=UrlBuilder.build_contest_url(), headers=self.headers)

        response.raise_for_status()

        return ContestsDeserializer.deserialize(contests_json=response.json())

    def get_fixture_lists(self):
        response = requests.get(url=UrlBuilder.build_fixture_lists_url(), headers=self.headers)

        response.raise_for_status()

        return FixtureListsDeserializer.deserialize(fixture_lists_json=response.json())

    def get_fixture_players(self, fixture_list_id):
        response = requests.get(url=UrlBuilder.build_fixture_list_players_url(fixture_list_id=fixture_list_id),
                                headers=self.headers)

        response.raise_for_status()

        return FixturePlayersDeserializer.deserialize(fixture_players_json=response.json())



from setuptools import setup, find_packages
setup(
  name='fan-duel-client',
  packages=find_packages(exclude=['tests*']),
  install_requires=['requests', 'enum34', 'pytz'],
  version='0.5',
  description='A FanDuel client',
  author='Jae Bradley',
  author_email='jae.b.bradley@gmail.com',
  url='https://github.com/jaebradley/fan_duel_client',
  download_url='https://github.com/jaebradley/fan_duel_client/tarball/0.3',
  keywords=['sports', 'dfs', 'fanduel'],
  classifiers=[],
)


from datetime import datetime

from pytz import utc

from fan_duel_client.data.models import Contest, FixtureListStatus, FixtureList, Sport, Team, Fixture, FixturePlayer,\
    FixturePlayerInjury, Position, FixtureStatus


def deserialize_start_time(start_time):
    assert isinstance(start_time, basestring)

    deserialized_start_time = datetime.strptime(start_time, FixtureListDeserializer.start_time_format)
    return utc.localize(deserialized_start_time)


class ContestsDeserializer:
    contests_field_name = 'contests'

    def __init__(self):
        pass

    @staticmethod
    def deserialize(contests_json):
        assert ContestsDeserializer.contests_field_name in contests_json

        return [ContestDeserializer.deserialize(contest_json=contest)
                for contest in contests_json[ContestsDeserializer.contests_field_name]]


class ContestDeserializer:
    contest_id_field_name = 'id'
    max_entries_per_user_field_name = 'max_entries_per_user'
    name_field_name = 'name'
    is_guaranteed_field_name = 'guaranteed'
    url_field_name = '_url'
    entry_fee_field_name = 'entry_fee'

    def __init__(self):
        pass

    @staticmethod
    def deserialize(contest_json):
        assert ContestDeserializer.contest_id_field_name in contest_json
        assert ContestDeserializer.max_entries_per_user_field_name in contest_json
        assert ContestDeserializer.name_field_name in contest_json
        assert ContestDeserializer.is_guaranteed_field_name in contest_json
        assert ContestDeserializer.url_field_name in contest_json
        assert ContestDeserializer.entry_fee_field_name in contest_json

        return Contest(contest_id=contest_json[ContestDeserializer.contest_id_field_name],
                       max_entries_per_user=contest_json[ContestDeserializer.max_entries_per_user_field_name],
                       name=contest_json[ContestDeserializer.name_field_name],
                       is_guaranteed=contest_json[ContestDeserializer.is_guaranteed_field_name],
                       url=contest_json[ContestDeserializer.url_field_name],
                       entry_fee=contest_json[ContestDeserializer.entry_fee_field_name])


class FixtureListStatusDeserializer:
    has_started_field_name = 'started'
    is_final_field_name = 'final'

    def __init__(self):
        pass

    @staticmethod
    def deserialize(fixture_list_status_json):
        assert FixtureListStatusDeserializer.has_started_field_name in fixture_list_status_json
        assert FixtureListStatusDeserializer.is_final_field_name in fixture_list_status_json

        return FixtureListStatus(has_started=fixture_list_status_json[FixtureListStatusDeserializer.has_started_field_name],
                                 is_final=fixture_list_status_json[FixtureListStatusDeserializer.is_final_field_name])


class FixtureListsDeserializer:
    fixtures_field_name = 'fixture_lists'

    def __init__(self):
        pass

    @staticmethod
    def deserialize(fixture_lists_json):
        assert FixtureListsDeserializer.fixtures_field_name in fixture_lists_json

        return [FixtureListDeserializer.deserialize(fixture_list_json=fixture)
                for fixture in fixture_lists_json[FixtureListsDeserializer.fixtures_field_name]]


class FixtureListDeserializer:
    fixture_id_field_name = 'id'
    url_field_name = '_url'
    status_field_name = 'status'
    start_time_field_name = 'start_date'
    players_field_name = 'players'
    players_url_field_name = '_url'
    sport_field_name = 'sport'
    salary_cap_field_name = 'salary_cap'

    start_time_timezone = utc
    start_time_format = '%Y-%m-%dT%H:%M:%SZ'

    def __init__(self):
        pass

    @staticmethod
    def deserialize(fixture_list_json):
        assert FixtureListDeserializer.fixture_id_field_name in fixture_list_json
        assert FixtureListDeserializer.url_field_name in fixture_list_json
        assert FixtureListDeserializer.status_field_name in fixture_list_json
        assert FixtureListDeserializer.start_time_field_name in fixture_list_json
        assert FixtureListDeserializer.players_field_name in fixture_list_json
        assert FixtureListDeserializer.players_url_field_name in fixture_list_json[FixtureListDeserializer.players_field_name]
        assert FixtureListDeserializer.sport_field_name in fixture_list_json
        assert FixtureListDeserializer.salary_cap_field_name in fixture_list_json

        return FixtureList(fixture_id=fixture_list_json[FixtureListDeserializer.fixture_id_field_name],
                           url=fixture_list_json[FixtureListDeserializer.url_field_name],
                           status=FixtureListStatusDeserializer.deserialize(fixture_list_status_json=fixture_list_json[FixtureListDeserializer.status_field_name]),
                           start_time=deserialize_start_time(start_time=fixture_list_json[FixtureListDeserializer.start_time_field_name]),
                           players_url=fixture_list_json[FixtureListDeserializer.players_field_name][FixtureListDeserializer.players_url_field_name],
                           sport=Sport.value_of(name=fixture_list_json[FixtureListDeserializer.sport_field_name]),
                           salary_cap=fixture_list_json[FixtureListDeserializer.salary_cap_field_name])


class FixtureStatusDeserializer:
    has_started_field_name = 'started'
    is_final_field_name = 'final'

    def __init__(self):
        pass

    @staticmethod
    def deserialize(fixture_status_json):
        assert FixtureStatusDeserializer.has_started_field_name in fixture_status_json
        assert FixtureStatusDeserializer.is_final_field_name in fixture_status_json

        return FixtureStatus(has_started=fixture_status_json[FixtureStatusDeserializer.has_started_field_name],
                             is_final=fixture_status_json[FixtureStatusDeserializer.is_final_field_name])


class FixtureTeamDeserializer:
    team_field_name = 'team'
    members_field_name = '_members'

    def __init__(self):
        pass

    @staticmethod
    def deserialize(team_json, sport):
        assert FixtureTeamDeserializer.team_field_name in team_json
        assert FixtureTeamDeserializer.members_field_name in team_json[FixtureTeamDeserializer.team_field_name]
        assert isinstance(sport, Sport)

        team_members = team_json[FixtureTeamDeserializer.team_field_name][FixtureTeamDeserializer.members_field_name]
        assert isinstance(team_members, list)
        assert len(team_members) == 1

        return Team.value_of(sport=sport, fan_duel_id=int(team_members[0]))


class FixtureDeserializer:
    fixture_id_field_name = 'id'
    sport_field_name = 'sport'
    status_field_name = 'status'
    away_team_field_name = 'away_team'
    home_team_field_name = 'home_team'
    start_time_field_name = 'start_date'

    def __init__(self):
        pass

    @staticmethod
    def deserialize(fixture_json):
        assert FixtureDeserializer.fixture_id_field_name in fixture_json
        assert FixtureDeserializer.sport_field_name in fixture_json
        assert FixtureDeserializer.status_field_name in fixture_json
        assert FixtureDeserializer.away_team_field_name in fixture_json
        assert FixtureDeserializer.home_team_field_name in fixture_json
        assert FixtureDeserializer.start_time_field_name in fixture_json

        sport = Sport.value_of(name=fixture_json[FixtureDeserializer.sport_field_name])
        away_team = FixtureTeamDeserializer.deserialize(team_json=fixture_json[FixtureDeserializer.away_team_field_name],
                                                        sport=sport)
        home_team = FixtureTeamDeserializer.deserialize(team_json=fixture_json[FixtureDeserializer.home_team_field_name],
                                                        sport=sport)
        status = FixtureStatusDeserializer.deserialize(fixture_status_json=fixture_json[FixtureDeserializer.status_field_name])
        start_time = deserialize_start_time(start_time=fixture_json[FixtureDeserializer.start_time_field_name])

        return Fixture(fixture_id=int(fixture_json[FixtureDeserializer.fixture_id_field_name]),
                       status=status, sport=sport, away_team=away_team, home_team=home_team, start_time=start_time)


class FixturePlayersDeserializer:
    fixtures_field_name = 'fixtures'
    players_field_name = 'players'

    def __init__(self):
        pass

    @staticmethod
    def deserialize(fixture_players_json):
        assert FixturePlayersDeserializer.fixtures_field_name in fixture_players_json
        assert FixturePlayersDeserializer.players_field_name in fixture_players_json

        raw_fixtures = fixture_players_json[FixturePlayersDeserializer.fixtures_field_name]
        assert isinstance(raw_fixtures, list)

        raw_players = fixture_players_json[FixturePlayersDeserializer.players_field_name]
        assert isinstance(raw_players, list)

        fixture_mapping = {}
        for raw_fixture in raw_fixtures:
            fixture = FixtureDeserializer.deserialize(fixture_json=raw_fixture)
            fixture_mapping[fixture.fixture_id] = fixture

        return [FixturePlayerDeserializer.deserialize(fixture_player_json=fixture_player, fixtures_mapping=fixture_mapping)
                for fixture_player in raw_players]


class FixturePlayerDeserializer:
    fixture_player_id_field_name = 'id'
    first_name_field_name = 'first_name'
    last_name_field_name = 'last_name'
    jersey_number_field_name = 'jersey_number'
    injury_details_field_name = 'injury_details'
    injury_status_field_name = 'injury_status'
    fixture_field_name = 'fixture'
    fixture_members_field_name = '_members'
    is_removed_field_name = 'removed'
    position_field_name = 'position'
    salary_field_name = 'salary'
    team_field_name = 'team'
    team_member_field_name = '_members'

    def __init__(self):
        pass

    @staticmethod
    def deserialize(fixture_player_json, fixtures_mapping):
        assert FixturePlayerDeserializer.fixture_player_id_field_name in fixture_player_json
        assert FixturePlayerDeserializer.first_name_field_name in fixture_player_json
        assert FixturePlayerDeserializer.last_name_field_name in fixture_player_json
        assert FixturePlayerDeserializer.jersey_number_field_name in fixture_player_json
        assert FixturePlayerDeserializer.injury_details_field_name in fixture_player_json
        assert FixturePlayerDeserializer.fixture_field_name in fixture_player_json
        assert FixturePlayerDeserializer.fixture_members_field_name in fixture_player_json[FixturePlayerDeserializer.fixture_field_name]
        assert FixturePlayerDeserializer.is_removed_field_name in fixture_player_json
        assert FixturePlayerDeserializer.position_field_name in fixture_player_json
        assert FixturePlayerDeserializer.salary_field_name in fixture_player_json
        assert FixturePlayerDeserializer.team_field_name in fixture_player_json
        assert FixturePlayerDeserializer.team_member_field_name in fixture_player_json[FixturePlayerDeserializer.team_field_name]

        fixture_members = fixture_player_json[FixturePlayerDeserializer.fixture_field_name][FixturePlayerDeserializer.fixture_members_field_name]
        assert isinstance(fixture_members, list)
        assert len(fixture_members) == 1

        fixture_id = int(fixture_members[0])
        fixture = fixtures_mapping.get(fixture_id)
        assert isinstance(fixture, Fixture)

        team_members = fixture_player_json[FixturePlayerDeserializer.team_field_name][FixturePlayerDeserializer.team_member_field_name]
        assert isinstance(team_members, list)
        assert len(team_members) == 1

        team_id = int(team_members[0])
        team = Team.value_of(sport=fixture.sport, fan_duel_id=team_id)

        injury = FixturePlayerInjury(status=fixture_player_json[FixturePlayerDeserializer.injury_status_field_name],
                                     details=fixture_player_json[FixturePlayerDeserializer.injury_details_field_name])

        return FixturePlayer(fixture_player_id=fixture_player_json[FixturePlayerDeserializer.fixture_player_id_field_name],
                             first_name=fixture_player_json[FixturePlayerDeserializer.first_name_field_name],
                             last_name=fixture_player_json[FixturePlayerDeserializer.last_name_field_name],
                             jersey_number=fixture_player_json[FixturePlayerDeserializer.jersey_number_field_name],
                             injury=injury,
                             is_removed=fixture_player_json[FixturePlayerDeserializer.is_removed_field_name],
                             position=Position.value_of(sport=fixture.sport,
                                                        abbreviation=fixture_player_json[FixturePlayerDeserializer.position_field_name]),
                             salary=fixture_player_json[FixturePlayerDeserializer.salary_field_name],
                             team=team,
                             fixture=fixture)
